@@ -4,6 +4,9 @@
             ,TypeFamilies
             ,TypeSynonymInstances #-}
 {-# OPTIONS -fno-warn-orphans #-}
+#if __GLASGOW_HASKELL__ > 901
+{-# OPTIONS -fno-warn-incomplete-uni-patterns #-}
+#endif
 
 {-
 Copyright (C) 2007 John Goerzen <jgoerzen@complete.org>
@@ -19,8 +22,8 @@ For license and copyright information, see the file COPYRIGHT
    Copyright  : Copyright (C) 2007 John Goerzen
    License    : BSD3
 
-   Maintainer : John Lato <jwlato@gmail.com>
-   Stability  : provisional
+   Maintainer : David Fox <dsf@seereason.com>, Andreas Abel
+   Stability  : stable
    Portability: portable
 
 Instances of 'Data.ListLike.ListLike' and related classes.
@@ -39,8 +42,6 @@ import Prelude hiding (length, head, last, null, tail, map, filter, concat,
                        splitAt, elem, notElem, unzip, lines, words,
                        unlines, unwords)
 import qualified Prelude as P
-import           Control.Applicative ((<$>), (<*>))
-import           Control.Monad
 import qualified Data.List as L
 import qualified Data.Sequence as S
 import           Data.Sequence ((><), (|>), (<|))
@@ -53,9 +54,9 @@ import           Data.ListLike.Text ()
 import           Data.ListLike.UTF8 ()
 import           Data.ListLike.Vector ()
 import           Data.Int
---import           Data.Maybe (fromMaybe)
-import           Data.Monoid
+#if !MIN_VERSION_base(4,11,0)
 import           Data.Semigroup (Semigroup(..))
+#endif
 import qualified Data.ByteString as BS
 import qualified Data.ByteString.Char8 as BSC
 --import qualified Data.Foldable as F
@@ -64,7 +65,7 @@ import qualified Data.Array.IArray as A
 import           Data.Array.IArray((!), (//), Ix(..))
 import qualified Data.ByteString.Lazy as BSL
 import qualified Data.ByteString.Lazy.Char8 as BSLC
-import           Data.String (IsString(fromString))
+import           Data.String (IsString)
 --import           Data.String.UTF8 (UTF8)
 --import qualified Data.String.UTF8 as UTF8
 import qualified System.IO as IO
@@ -132,10 +133,12 @@ instance FoldableLL BS.ByteString Word8 where
     foldr' = BS.foldr'
     foldr1 = BS.foldr1
 
+#if !MIN_VERSION_bytestring(0,10,12)
 instance IsList BS.ByteString where
     type Item BS.ByteString = Word8
     toList = BS.unpack
     fromList = BS.pack
+#endif
 
 instance ListLike BS.ByteString Word8 where
     empty = BS.empty
@@ -187,8 +190,8 @@ instance ListLike BS.ByteString Word8 where
     findIndices x = fromList . BS.findIndices x
     -- the default definitions don't work well for array-like things, so
     -- do monadic stuff via a list instead
-    sequence  = liftM fromList . P.sequence  . toList
-    mapM func = liftM fromList . P.mapM func . toList
+    sequence  = fmap fromList . P.sequenceA  . toList
+    mapM func = fmap fromList . P.traverse func . toList
     --nub = BS.nub
     --delete = BS.delete
     --deleteFirsts = BS.deleteFirsts
@@ -255,10 +258,12 @@ mi64toi :: Maybe Int64 -> Maybe Int
 mi64toi Nothing = Nothing
 mi64toi (Just x) = Just (fromIntegral x)
 
+#if !MIN_VERSION_bytestring(0,10,12)
 instance IsList BSL.ByteString where
     type Item BSL.ByteString = Word8
     toList = BSL.unpack
     fromList = BSL.pack
+#endif
 
 instance ListLike BSL.ByteString Word8 where
     empty = BSL.empty
@@ -308,8 +313,8 @@ instance ListLike BSL.ByteString Word8 where
     --elemIndices x = fromList . L.map fromIntegral . BSL.elemIndices x
     findIndex f = mi64toi . BSL.findIndex f
     --findIndices x = fromList . L.map fromIntegral . BSL.findIndices x
-    sequence  = liftM fromList . P.sequence  . toList
-    mapM func = liftM fromList . P.mapM func . toList
+    sequence  = fmap fromList . P.sequenceA  . toList
+    mapM func = fmap fromList . P.traverse func . toList
     --sequence = BSL.sequence
     --mapM = BSL.mapM
     --mapM_ = BSL.mapM_
@@ -384,17 +389,17 @@ instance Ix i => FoldableLL (A.Array i e) e where
     foldr' = F.foldr'
 
 instance (Integral i, Ix i) => Semigroup (A.Array i e) where
-  (<>) = mappend
+  l1 <> l2 = A.array (blow, newbhigh) $
+               A.assocs l1 ++ zip [bhigh + 1 .. newbhigh] (A.elems l2)
+    where
+    newlen        = genericLength newelems
+    newelems      = A.elems l2
+    newbhigh      = bhigh + newlen
+    (blow, bhigh) = A.bounds l1
 
 instance (Integral i, Ix i) => Monoid (A.Array i e) where
-    mempty = A.listArray (0, -1) []
-    mappend l1 l2 =
-        A.array (blow, newbhigh)
-              (A.assocs l1 ++ zip [(bhigh + 1)..newbhigh] (A.elems l2))
-        where newlen = genericLength newelems
-              newelems = A.elems l2
-              newbhigh = bhigh + newlen
-              (blow, bhigh) = A.bounds l1
+  mempty  = A.listArray (0, -1) []
+  mappend = (<>)
 
 instance (Integral i, Ix i) => IsList (A.Array i e) where
     type Item (A.Array i e) = e
@@ -457,8 +462,8 @@ instance (Integral i, Ix i) => ListLike (A.Array i e) e where
     elemIndices i = fromList . L.elemIndices i . toList
     findIndex f = L.findIndex f . toList
     findIndices f = fromList . L.findIndices f . toList
-    sequence  = liftM fromList . P.sequence  . toList
-    mapM func = liftM fromList . P.mapM func . toList
+    sequence  = fmap fromList . P.sequenceA  . toList
+    mapM func = fmap fromList . P.traverse func . toList
     -- rigidMapM = mapM
     nub = fromList . L.nub . toList
     -- delete
