@@ -6,6 +6,7 @@
             ,FunctionalDependencies
             ,FlexibleInstances
             ,UndecidableInstances
+            ,TypeFamilies
             ,FlexibleContexts #-}
 {-# OPTIONS -fno-warn-orphans #-}
 
@@ -22,7 +23,7 @@ For license and copyright information, see the file COPYRIGHT
 module TestInfrastructure where
 
 import Test.QuickCheck
---import Test.QuickCheck.Test
+import Test.QuickCheck.Test
 import qualified Data.ByteString as BS
 import qualified Data.ByteString.Lazy as BSL
 import qualified Data.ListLike as LL
@@ -45,9 +46,11 @@ import System.IO
 import qualified Test.HUnit as HU
 import Text.Printf
 import Data.Function (on)
+import Data.String (IsString(fromString))
 import Data.Word
 import Data.List
 import Data.Monoid (Monoid(..))
+import GHC.Exts (IsList(Item))
 
 simpleArb :: (LL.ListLike f i, Arbitrary i) => Gen f
 simpleArb = sized (\n -> choose (0, n) >>= myVector)
@@ -273,15 +276,22 @@ instance Monoid (MyList a) where
     mempty = MyList []
     mappend (MyList x) (MyList y) = MyList (x ++ y)
 
+instance IsList (MyList a) where
+    type Item (MyList a) = a
+    toList (MyList xs) = xs
+    fromList = MyList
+
 instance LL.ListLike (MyList a) a where
     singleton x = MyList [x]
     head (MyList x) = head x
     tail (MyList x) = MyList (tail x)
     null (MyList x) = null x
 
+instance IsString (MyList Char) where
+    fromString = MyList
+
 instance LL.StringLike (MyList Char) where
     toString (MyList x) = x
-    fromString x = MyList x
 
 mkTest :: Testable prop => String -> prop -> HU.Test
 mkTest msg test = HU.TestLabel msg $ HU.TestCase (quickCheckResult test >>= HU.assertBool msg . isSuccess)
@@ -390,7 +400,7 @@ aps msg x = HU.TestLabel msg $ HU.TestList $
      w "Text.Lazy" (x::LLTest TL.Text Char),
      w "Text.Builder" (x::LLTest TB.Builder Char),
      w "UTF8 ByteString" (x::LLTest (UTF8.UTF8 BS.ByteString) Char),
-     w "UTF8 ByteString.Lazy" (x::LLTest (UTF8.UTF8 BSL.ByteString) Char)
-    ,w "Vector Char" (x::LLTest (V.Vector Char) Char),
+     w "UTF8 ByteString.Lazy" (x::LLTest (UTF8.UTF8 BSL.ByteString) Char),
+     w "Vector Char" (x::LLTest (V.Vector Char) Char),
      w "Vector.Unbox Char" (x::LLTest (VU.Vector Char) Char)
     ]
